<?php
namespace PrestaShop\Module\PolarisPrestaConnector;

use PolarisPrestaConnector;

class Upgrade {

    /**
     * Déclenche la mise à jour de la base de données du module
     */
    public function upgradeModule(PolarisPrestaConnector $module, string $version)
    {
        $doing = "init";
        try {
            // A partir d'ici, capturer toute les sorties PHP
            ob_start();

            // On scanne le répertoire de ce fichier à la recherche de tous les fichiers upgrade-{$version}.php possible
            // on trie les fichiers par ordre croissant de version qu'il représente
            $files = glob(__DIR__ . '/upgrade-*.php');
            usort($files, function($a, $b) {
                $a = explode('-', basename($a));
                $b = explode('-', basename($b));
                return version_compare($a[1], $b[1]);
            });
            
            // Pour chaque fichier dont la version est supérieure à la version actuelle du module ($version)
            // on l'inclut et on appelle la méthode upgrade
            $db = \Db::getInstance();

            foreach ($files as $file) {
                $fileVersion = explode('-', basename($file, '.php'))[1];
                $doing = $fileVersion;
                if (version_compare($fileVersion, $version) > 0) {                
                    require_once $file;
                    // La classe est le nom de la version avec _ à la place des .
                    $classname = '\\PrestaShop\\Module\\PolarisPrestaConnector\\Upgrade_' . str_replace('.', '_', $fileVersion);
                    $upgrade = new $classname();
                    $upgrade->upgrade($module, $db);

                    // Puis on met à jour la version du module dans notre configuration
                    \Configuration::updateValue($module->BridgePrefix . 'prestaconnector_version', $fileVersion);
                }
            }

            // Et maintenant, on applique la configuration par défaut
            $module->getCfg()->applyDefaults();

            // On réenregistre les hooks
            $module->registerHooks(true);
        }
        catch (\Exception $e) {
            throw new \Exception("Erreur à la mise à jour du fichier ".$doing." : ".$e->getMessage());
        }
        finally {
            // On vide le buffer de sortie dans une variable
            $output = ob_get_clean();

            if (!empty(trim($output))) 
            {
                // On affiche le contenu du buffer dans les journaux Prestashop
                // En tant qu'erreur
                \PrestaShopLogger::addLog("Erreur lors de la mise à jour de la DB : ".$output, 4, null, $module->name);
            }

            // Et on rétablit la sortie normale
            ob_end_clean();            
        }        
    }
}