<?php

namespace PrestaShop\Module\PolarisPrestaConnector\Forms;

use PolarisPrestaConnector;
use PrestaShop\Module\PolarisPrestaConnector\SyncConfiguration;
use PrestaShopBundle\Form\Admin\Type\CustomContentType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\Form\AbstractType;
use PrestaShopBundle\Form\Admin\Type\SwitchType;
use Symfony\Component\Form\Extension\Core\Type\CheckboxType;
use Symfony\Component\Form\Extension\Core\Type\ChoiceType;
use Symfony\Component\Form\Extension\Core\Type\SubmitType;
use Symfony\Component\Form\Extension\Core\Type\TextType;

class catalogType extends AbstractType {

    /**
     * lien vers le module
     */
    protected PolarisPrestaConnector $module;

    public function __construct() {
        $this->module = \Module::getInstanceByName(strtolower('PolarisPrestaConnector'));
    }

    public function buildForm(FormBuilderInterface $builder, array $options)
    {
        $backofficeName = $this->module->getBridge()->getId();
        $cfg = $this->module->getCfg();

        $builder->add('title_stock', CustomContentType::class, [
            'label' => false,
            'template' => '@Modules/prestaconnector/views/templates/admin/form_title.html.twig',
            'data' => [
                'hr' => false,
                'title' => 'Options de stock',
                'help' => '',
            ],
        ]);

        /**
         * Stock de sécurité
         */
        $builder
            ->add('secured_stock', TextType::class, [
                'label' => 'Stock de sécurité',                
                'required' => false,
                'help' => 'Cette valeur est déduite du stock de chaque taille de produit lors de l\'importation du stock.', // should be set to enable help block
                'data' => $this->module->getCfg()->get(SyncConfiguration::CFG_SECURED_STOCK),
            ]);

        /**
         * Cacher/montrer les produits en stock
         */
        $builder
            ->add('hide_stock_method', ChoiceType::class, [
                'label' => 'Visibilité des produits en stock',                
                //'choice_attr' => $this->productVisibilityChoiceProvider->getChoicesAttributes(),
                'choices' => [
                    'Ne pas changer la visibilité du produit' => '',
                    'Produit visible' => 'both',
                    'Cacher le produit' => 'none',
                    'Produit uniquement visible dans le parcours catalogue' => 'catalog',
                    'Produit uniquement visible lors d\'une recherche' => 'search',
                ],

                'expanded' => false,
                'required' => false,
                // placeholder false is important to avoid empty option in radio select despite required being false
                'placeholder' => false,
                'column_breaker' => true,
                'help' => '', // should be set to enable help block
                'data' => $this->module->getCfg()->get(SyncConfiguration::CFG_NO_STOCK_PRODUCT_VISIBILITY ),
            ]);        

        /**
         * CFG_PR_ATTRIBUTES_MAPPING
         */
        $productMapping = $cfg->get(SyncConfiguration::CFG_PR_ATTRIBUTES_MAPPING);
        $productAttrs = $cfg->get(SyncConfiguration::CFG_DYN_ATTR);
        $productAttrsExtended = array_merge($productAttrs,$cfg->get(SyncConfiguration::CFG_DYN_CAT));

        /** Mapping des produits */
        $builder->add('title_product_mapping', CustomContentType::class, [
            'label' => false,
            'template' => '@Modules/prestaconnector/views/templates/admin/form_title.html.twig',
            'data' => [
                'hr' => true,
                'title' => 'Attributs discriminants',
                'help' => '<p>
                                Sélectionnez les attributs '.$backofficeName.' que vous souhaitez utiliser comme discriminants de produit Prestashop (cf. documentation).<br>
                                <em>Par exemple, si vous désirez créer un produit Prestashop par couleur, activez "couleur".</em>
                            </p>',
            ],
        ]);

        // Création dynamique des champs en fonction des options de configuration
        foreach ($productAttrs as $k => $name)
        {
            $safe_key = 'PRODUCT_ATTR_PRODUCT_'.trim(base64_encode($k), '=');
            $builder->add($safe_key, SwitchType::class, [
                'label' => $name,
                'required' => false,
                'data' => $productMapping[SyncConfiguration::PRODUCT_ATTR_PRODUCT][$k] ?? false, // valeur par défaut selon l'option configurée
            ]);
        }

        /** Mapping des attributs */
        $builder->add('title_attribute_mapping', CustomContentType::class, [
            'label' => false,
            'template' => '@Modules/prestaconnector/views/templates/admin/form_title.html.twig',
            'data' => [
                'hr' => true,
                'title' => 'Mappage des attributs de produit',
                'help' => '<p>
                                Pour chaque type d\'attributs de votre back-office '.$backofficeName.' sélectionnez le <strong>groupe d\'attributs Prestashop</strong> dans lequel vous souhaitez reporter la valeur.
                                <br>
                                Si vous n\'indiquez rien, l\'attribut ne sera pas synchronisé dans un groupe d\'attributs.
                            </p>',
            ],
        ]);

        // Création dynamique des champs en fonction des options de configuration
        $attributeGroups = ['---' => 0];

        // Pour empêcher les erreurs, on retire tous les groupes d'attributs qui ont été affectés à des tailles
        $sizeGrids = $cfg->get(SyncConfiguration::CFG_MAP_SIZEGRIDS);

        foreach (\AttributeGroup::getAttributesGroups(\Context::getContext()->language->id) as $ag)
            if (!in_array($ag['id_attribute_group'], $sizeGrids))
                $attributeGroups[$ag['name']] = $ag['id_attribute_group'];

        foreach ($productAttrsExtended as $k => $name)
        {
            $safe_key = 'PRODUCT_ATTR_VARIANT_'.trim(base64_encode($k), '=');
            $builder->add($safe_key, ChoiceType::class, [
                'label' => $name,
                'required' => false,
                'expanded' => false,
                'multiple' => false,
                'empty_data' => 0,
                'choices' => $attributeGroups,
                'data' => $productMapping[SyncConfiguration::PRODUCT_ATTR_VARIANT][$k] ?? 0, // valeur par défaut selon l'option configurée
            ]);
        }

        /** Mapping des caractéristiques */
        $builder->add('title_features_mapping', CustomContentType::class, [
            'label' => false,
            'template' => '@Modules/prestaconnector/views/templates/admin/form_title.html.twig',
            'data' => [
                'hr' => true,
                'title' => 'Mappage des caractéristiques de produit',
                'help' => '<p>
                                Pour chaque type d\'attributs de votre back-office '.$backofficeName.' sélectionnez la <strong>caractéristique Prestashop</strong> dans laquelle vous souhaitez reporter la valeur.
                                <br>
                                Si vous n\'indiquez rien, l\'attribut ne sera pas synchronisé dans une caractéristique.
                            </p>',
            ],
        ]);

        // Création dynamique des champs en fonction des options de configuration
        $features = ['---' => 0];
        foreach (\Feature::getFeatures(\Context::getContext()->language->id) as $feat)
            $features[$feat['name']] = $feat['id_feature'];

        foreach ($productAttrsExtended as $k => $name)
        {
            $safe_key = 'PRODUCT_ATTR_FEATURE_'.trim(base64_encode($k), '=');
            $builder->add($safe_key, ChoiceType::class, [
                'label' => $name,
                'required' => false,
                'expanded' => false,
                'multiple' => false,
                'empty_data' => 0,
                'choices' => $features,
                'data' => $productMapping[SyncConfiguration::PRODUCT_ATTR_FEATURE][$k] ?? 0, // valeur par défaut selon l'option configurée
            ]);
        }

        /**
         * Options de synchronisation des fiches produits
         */
        $builder->add('title_sync', CustomContentType::class, [
            'label' => false,
            'template' => '@Modules/prestaconnector/views/templates/admin/form_title.html.twig',
            'data' => [
                'hr' => true,
                'title' => 'Données produits synchronisées',
                'help' => '<p>Sélectionnez ici les informations que vous désirez synchroniser depuis votre back-office Polaris vers votre boutique en ligne :</p>
                            <ul>
                                <li>les informations sélectionnées sont systématiquement remplacées par celles de votre back-office lors de la synchronisation du produit ;</li>
                                <li>celles qui ne le sont pas sont uniquement positionnées lors de la création du produit.</li>
                            </ul>
                            ',
            ],
        ]);
        
        $cfgOptions = $cfg->get(SyncConfiguration::CFG_DATA_SYNC);        
        $optionsDesc = $cfg->describeSyncConfiguration();
        $mode = $cfg->get(SyncConfiguration::CFG_WORKMODE);

        // Création dynamique des champs en fonction des options de configuration
        foreach ($optionsDesc as $field => $p)
        {
            $isActive = isset($cfgOptions[$field]) ? $cfgOptions[$field] : false;
            $builder->add($field, SwitchType::class, [
                'label' => $p['label'],
                //'help' => $p['description'],  // Utilisation de la description pour l'aide                
                'required' => false,
                'disabled' => $mode === 0,
                'data' => $isActive, // valeur par défaut selon l'option configurée
            ]);
        }

        // Bouton de sauvegarde
        $builder->add('sauvegarder', SubmitType::class, [
            'attr' => ['class' => 'btn-form-valid btn-primary']
        ]);
    }
}